@@ -1,23 +1,10 @@
-/*
- * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
- *
- * Author:     zccrs <zccrs@live.com>
- *
- * Maintainer: zccrs <zhangjide@deepin.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 - 2022 Uniontech Software Technology Co.,Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
+#define protected public
+#include <QWindow>
+#undef protected
 #include "dnotitlebarwindowhelper.h"
 #include "vtablehook.h"
 #include "utility.h"
@@ -25,9 +12,6 @@
 #include "dnativesettings.h"
 #include "dplatformintegration.h"
 
-#define protected public
-#include <QWindow>
-#undef protected
 #include <QMouseEvent>
 #include <QTimer>
 #include <QMetaProperty>
@@ -39,10 +23,14 @@
 #define _DEEPIN_SCISSOR_WINDOW "_DEEPIN_SCISSOR_WINDOW"
 Q_DECLARE_METATYPE(QPainterPath)
 
+extern QWidget *qt_button_down;
+
 DPP_BEGIN_NAMESPACE
 
 QHash<const QWindow*, DNoTitlebarWindowHelper*> DNoTitlebarWindowHelper::mapped;
 
+static QHash<DNoTitlebarWindowHelper*, QPointF> g_pressPoint;
+
 DNoTitlebarWindowHelper::DNoTitlebarWindowHelper(QWindow *window, quint32 windowID)
     : QObject(window)
     , m_window(window)
@@ -66,6 +54,8 @@ DNoTitlebarWindowHelper::DNoTitlebarWindowHelper(QWindow *window, quint32 window
         updateShadowRadiusFromProperty();
         updateShadowOffsetFromProperty();
         updateShadowColorFromProperty();
+        updateWindowEffectFromProperty();
+        updateWindowStartUpEffectFromProperty();
         updateEnableSystemResizeFromProperty();
         updateEnableSystemMoveFromProperty();
         updateEnableBlurWindowFromProperty();
@@ -73,7 +63,7 @@ DNoTitlebarWindowHelper::DNoTitlebarWindowHelper(QWindow *window, quint32 window
         updateWindowBlurPathsFromProperty();
         updateAutoInputMaskByClipPathFromProperty();
     } else {
-        qWarning() << "native settings is invalid for window: 0x" << hex << windowID;
+        qWarning() << "native settings is invalid for window: 0x" << /*hex <<*/ windowID;
     }
 
     connect(DWMSupport::instance(), &DXcbWMSupport::hasScissorWindowChanged, this, &DNoTitlebarWindowHelper::updateWindowShape);
@@ -89,11 +79,13 @@ DNoTitlebarWindowHelper::DNoTitlebarWindowHelper(QWindow *window, quint32 window
 
 DNoTitlebarWindowHelper::~DNoTitlebarWindowHelper()
 {
+    g_pressPoint.remove(this);
+
     if (VtableHook::hasVtable(m_window)) {
         VtableHook::resetVtable(m_window);
     }
 
-    mapped.remove(qobject_cast<QWindow*>(parent()));
+    mapped.remove(static_cast<QWindow*>(parent()));
 
     if (m_window->handle()) { // 当本地窗口还存在时，移除设置过的窗口属性
         Utility::clearWindowProperty(m_windowID, Utility::internAtom(_DEEPIN_SCISSOR_WINDOW));
@@ -147,7 +139,7 @@ static QPair<qreal, qreal> takePair(const QVariant &value, const QPair<qreal, qr
         return defaultValue;
     }
 
-    const QStringList &l = value.toStringList();
+    const QStringList &l = value.toString().split(',');
 
     if (l.count() < 2) {
         return defaultValue;
@@ -167,7 +159,7 @@ static QMarginsF takeMargins(const QVariant &value, const QMarginsF &defaultValu
         return defaultValue;
     }
 
-    const QStringList &l = value.toStringList();
+    const QStringList &l = value.toString().split(',');
 
     if (l.count() < 4) {
         return defaultValue;
@@ -209,7 +201,7 @@ qreal DNoTitlebarWindowHelper::shadowRadius() const
 
 QPointF DNoTitlebarWindowHelper::shadowOffset() const
 {
-    return toPos(takePair(property("shadowOffect"), qMakePair(0.0, 0.0)));
+    return toPos(takePair(property("shadowOffset"), qMakePair(0.0, 0.0)));
 }
 
 QColor DNoTitlebarWindowHelper::shadowColor() const
@@ -217,6 +209,16 @@ QColor DNoTitlebarWindowHelper::shadowColor() const
     return qvariant_cast<QColor>(property("shadowColor"));
 }
 
+quint32 DNoTitlebarWindowHelper::windowEffect()
+{
+    return qvariant_cast<quint32>(property("windowEffect"));
+}
+
+quint32 DNoTitlebarWindowHelper::windowStartUpEffect()
+{
+    return qvariant_cast<quint32>(property("windowStartUpEffect"));
+}
+
 QMarginsF DNoTitlebarWindowHelper::mouseInputAreaMargins() const
 {
     return takeMargins(property("mouseInputAreaMargins"), QMarginsF(0, 0, 0, 0));
@@ -258,7 +260,7 @@ void DNoTitlebarWindowHelper::setShadowRadius(qreal shadowRadius)
 
 void DNoTitlebarWindowHelper::setShadowOffect(const QPointF &shadowOffect)
 {
-    setProperty("shadowOffect", QString("%1,%2").arg(shadowOffect.x()).arg(shadowOffect.y()));
+    setProperty("shadowOffset", QString("%1,%2").arg(shadowOffect.x()).arg(shadowOffect.y()));
 }
 
 void DNoTitlebarWindowHelper::setShadowColor(const QColor &shadowColor)
@@ -272,6 +274,16 @@ void DNoTitlebarWindowHelper::setMouseInputAreaMargins(const QMarginsF &mouseInp
                                                                .arg(mouseInputAreaMargins.right()).arg(mouseInputAreaMargins.bottom()));
 }
 
+void DNoTitlebarWindowHelper::setWindowEffect(quint32 effectScene)
+{
+    setProperty("windowEffect", effectScene);
+}
+
+void DNoTitlebarWindowHelper::setWindowStartUpEffect(quint32 effectType)
+{
+    setProperty("windowStartUpEffect", effectType);
+}
+
 void DNoTitlebarWindowHelper::updateClipPathFromProperty()
 {
     const QVariant &v = m_window->property(clipPath);
@@ -372,6 +384,30 @@ void DNoTitlebarWindowHelper::updateShadowColorFromProperty()
     }
 }
 
+void DNoTitlebarWindowHelper::updateWindowEffectFromProperty()
+{
+    const QVariant &v = m_window->property("_d_windowEffect");
+    const quint32 effectScene = qvariant_cast<quint32>(v);
+
+    if (effectScene) {
+        setWindowEffect(effectScene);
+    } else {
+        resetProperty("windowEffect");
+    }
+}
+
+void DNoTitlebarWindowHelper::updateWindowStartUpEffectFromProperty()
+{
+    const QVariant &v = m_window->property("_d_windowStartUpEffect");
+    const quint32 effectType = qvariant_cast<quint32>(v);
+
+    if (effectType) {
+        setWindowStartUpEffect(effectType);
+    } else {
+        resetProperty("windowStartUpEffect");
+    }
+}
+
 void DNoTitlebarWindowHelper::updateEnableSystemResizeFromProperty()
 {
     const QVariant &v = m_window->property(enableSystemResize);
@@ -425,7 +461,21 @@ void DNoTitlebarWindowHelper::updateWindowBlurAreasFromProperty()
 {
     const QVariant &v = m_window->property(windowBlurAreas);
     const QVector<quint32> &tmpV = qvariant_cast<QVector<quint32>>(v);
-    const QVector<Utility::BlurArea> &a = *(reinterpret_cast<const QVector<Utility::BlurArea>*>(&tmpV));
+    const int OffSet = 6; // Utility::BlurArea's member variables
+
+    Q_ASSERT(tmpV.size() % OffSet == 0);
+
+    QVector<Utility::BlurArea> a;
+    for (int i = 0; i < tmpV.size(); i += OffSet) {
+        Utility::BlurArea area;
+        area.x = tmpV[i + 0];
+        area.y = tmpV[i + 1];
+        area.width = tmpV[i + 2];
+        area.height = tmpV[i + 3];
+        area.xRadius = tmpV[i + 4];
+        area.yRaduis = tmpV[i + 5];
+        a << area;
+    }
 
     if (a.isEmpty() && m_blurAreaList.isEmpty())
         return;
@@ -464,6 +514,12 @@ bool DNoTitlebarWindowHelper::windowEvent(QEvent *event)
 {
     QWindow *w = this->window();
 
+    // TODO Crashed when delete by Vtable.
+    if (event->type() == QEvent::DeferredDelete) {
+        VtableHook::resetVtable(w);
+        return w->event(event);
+    }
+
     // get touch begin position
     static bool isTouchDown = false;
     static QPointF touchBeginPosition;
@@ -492,6 +548,7 @@ bool DNoTitlebarWindowHelper::windowEvent(QEvent *event)
     if (event->type() == QEvent::MouseButtonRelease) {
         self->m_windowMoving = false;
         Utility::updateMousePointForWindowMove(winId, true);
+        g_pressPoint.remove(this);
     }
 
     if (is_mouse_move && self->m_windowMoving) {
@@ -500,25 +557,35 @@ bool DNoTitlebarWindowHelper::windowEvent(QEvent *event)
 
     bool ret = VtableHook::callOriginalFun(w, &QWindow::event, event);
 
-    // workaround for kwin: Qt receives no release event when kwin finishes MOVE operation, 
+    // workaround for kwin: Qt receives no release event when kwin finishes MOVE operation,
     // which makes app hang in windowMoving state. when a press happens, there's no sense of
     // keeping the moving state, we can just reset ti back to normal.
     if (event->type() == QEvent::MouseButtonPress) {
         self->m_windowMoving = false;
+        g_pressPoint[this] = dynamic_cast<QMouseEvent*>(event)->globalPos();
     }
 
-    if (is_mouse_move && !event->isAccepted()) {
+    if (is_mouse_move && !event->isAccepted() && g_pressPoint.contains(this)) {
         QMouseEvent *me = static_cast<QMouseEvent*>(event);
         QRect windowRect = QRect(QPoint(0, 0), w->size());
         if (!windowRect.contains(me->windowPos().toPoint())) {
             return ret;
         }
 
+        QPointF delta = me->globalPos() - g_pressPoint[this];
+        if (delta.manhattanLength() < QGuiApplication::styleHints()->startDragDistance()) {
+            return ret;
+        }
+
         if (!self->m_windowMoving && self->isEnableSystemMove(winId)) {
             self->m_windowMoving = true;
 
             event->accept();
             startMoveWindow(winId);
+
+            if (qt_button_down) {
+                qt_button_down = nullptr;
+            }
         }
     }
 
