@@ -1,19 +1,10 @@
-/*
- * Copyright (C) 2017 ~ 2018 Deepin Technology Co., Ltd.
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 - 2022 Uniontech Software Technology Co.,Ltd.
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
+
+#define private public
+#include <QGuiApplication>
+#undef private
 
 #include "dplatformintegration.h"
 #include "global.h"
@@ -50,7 +41,6 @@
 #include "dhighdpi.h"
 
 #include "qxcbscreen.h"
-#include "qxcbbackingstore.h"
 
 #include <X11/cursorfont.h>
 #include <xcb/xcb_image.h>
@@ -80,6 +70,8 @@
 #define XSETTINGS_CURSOR_BLINK_TIME QByteArrayLiteral("Net/CursorBlinkTime")
 #define XSETTINGS_DOUBLE_CLICK_TIME QByteArrayLiteral("Net/DoubleClickTime")
 
+Q_LOGGING_CATEGORY(lcDxcb, "dtk.qpa.dxcb", QtInfoMsg)
+
 class DQPaintEngine : public QPaintEngine
 {
 public:
@@ -123,6 +115,8 @@ DPlatformIntegration::DPlatformIntegration(const QStringList &parameters, int &a
 
 DPlatformIntegration::~DPlatformIntegration()
 {
+    sendEndStartupNotifition();
+
 #ifdef Q_OS_LINUX
     if (m_eventFilter) {
         qApp->removeNativeEventFilter(m_eventFilter);
@@ -152,15 +146,7 @@ void DPlatformIntegration::setWindowProperty(QWindow *window, const char *name,
 
 bool DPlatformIntegration::enableDxcb(QWindow *window)
 {
-    static bool xwayland = QByteArrayLiteral("wayland") == qgetenv("XDG_SESSION_TYPE")
-            && !qEnvironmentVariableIsEmpty("WAYLAND_DISPLAY");
-
-    if (xwayland) {
-        // for xwayland
-        return false;
-    }
-
-    qDebug() << __FUNCTION__ << window << window->type() << window->parent();
+    qCDebug(lcDxcb) << "window:" << window << "window type:" << window->type() << "parent:" << window->parent();
 
     if (window->type() == Qt::Desktop)
         return false;
@@ -225,7 +211,8 @@ bool DPlatformIntegration::setEnableNoTitlebar(QWindow *window, bool enable)
     if (enable && DNoTitlebarWindowHelper::mapped.value(window))
         return true;
 
-    qDebug() << __FUNCTION__ << enable << window << window->type() << window->parent();
+    qCDebug(lcDxcb) << "enable titlebar:" << enable << "window:" << window
+    << "window type:" << window->type() << "parent:" << window->parent();
 
     if (enable) {
         if (window->type() == Qt::Desktop)
@@ -299,7 +286,7 @@ void DPlatformIntegration::setWMClassName(const QByteArray &name)
 
 QPlatformWindow *DPlatformIntegration::createPlatformWindow(QWindow *window) const
 {
-    qDebug() << __FUNCTION__ << window << window->type() << window->parent();
+    qCDebug(lcDxcb) << "window:" << window << "window type:" << window->type() << "parent:" << window->parent();
 
     if (qEnvironmentVariableIsSet("DXCB_PRINT_WINDOW_CREATE")) {
         printf("New Window: %s(0x%llx, name: \"%s\")\n", window->metaObject()->className(), (quintptr)window, qPrintable(window->objectName()));
@@ -425,7 +412,7 @@ QPlatformWindow *DPlatformIntegration::createPlatformWindow(QWindow *window) con
 
 QPlatformBackingStore *DPlatformIntegration::createPlatformBackingStore(QWindow *window) const
 {
-    qDebug() << __FUNCTION__ << window << window->type() << window->parent();
+    qCDebug(lcDxcb) << "window:" << window << "window type:" << window->type() << "parent:" << window->parent();
 
     QPlatformBackingStore *store = DPlatformIntegrationParent::createPlatformBackingStore(window);
     bool useGLPaint = DBackingStoreProxy::useGLPaint(window);
@@ -472,7 +459,7 @@ QPaintEngine *DPlatformIntegration::createImagePaintEngine(QPaintDevice *paintDe
     static QPaintEngine::PaintEngineFeatures disable_features = QPaintEngine::PaintEngineFeatures(-1);
 
     if (int(disable_features) < 0) {
-        disable_features = 0;
+        disable_features = QPaintEngine::PaintEngineFeatures();
 
         QByteArray data = qgetenv("DXCB_PAINTENGINE_DISABLE_FEATURES");
 
@@ -484,12 +471,14 @@ QPaintEngine *DPlatformIntegration::createImagePaintEngine(QPaintDevice *paintDe
                 if (ok)
                     break;
 
-                disable_features = 0;
+                disable_features = QPaintEngine::PaintEngineFeatures();
             }
 
             QSettings settings(QSettings::IniFormat, QSettings::UserScope, "deepin", "qt-theme");
 
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
             settings.setIniCodec("utf-8");
+#endif
             settings.beginGroup("Platform");
 
             bool ok = false;
@@ -497,7 +486,7 @@ QPaintEngine *DPlatformIntegration::createImagePaintEngine(QPaintDevice *paintDe
             disable_features = QPaintEngine::PaintEngineFeatures(settings.value("PaintEngineDisableFeatures").toByteArray().toInt(&ok, 16));
 
             if (!ok)
-                disable_features = 0;
+                disable_features = QPaintEngine::PaintEngineFeatures();
         } while (false);
     }
 
@@ -547,6 +536,8 @@ QVariant DPlatformIntegration::styleHint(QPlatformIntegration::StyleHint hint) c
     case MouseDoubleClickInterval:
         GET_VALID_XSETTINGS(XSETTINGS_DOUBLE_CLICK_TIME);
         break;
+    case ShowShortcutsInContextMenus:
+        return false;
     default:
         break;
     }
@@ -723,7 +714,8 @@ static xcb_cursor_t qt_xcb_createCursorXRender(QXcbScreen *screen, const QImage
         free(formatsReply);
         return XCB_NONE;
     }
-    memcpy(xi->data, img.constBits(), img.byteCount());
+
+    memcpy(xi->data, img.constBits(), img.sizeInBytes());
 
     xcb_pixmap_t pix = xcb_generate_id(conn);
     xcb_create_pixmap(conn, 32, pix, screen->root(), w, h);
@@ -850,7 +842,11 @@ static xcb_cursor_t overrideCreateFontCursor(QXcbCursor *xcb_cursor, QCursor *c,
         // special case for non-standard dnd-* cursors
         cursor = loadCursor(dpy, cshape);
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+        if (!cursor && !xcb_cursor->m_callbackForPropertyRegistered) {
+#else
         if (!cursor && !xcb_cursor->m_gtkCursorThemeInitialized) {
+#endif
             VtableHook::callOriginalFun(xcb_cursor, &QPlatformCursor::changeCursor, c, window);
 
             if (updateCursorTheme(dpy)) {
@@ -991,25 +987,46 @@ static void startDrag(QXcbDrag *drag)
     const Qt::DropActions actions = drag->currentDrag()->supportedActions();
 
     if (actions.testFlag(Qt::CopyAction))
-        support_actions << drag->atom(QXcbAtom::XdndActionCopy);
+        support_actions << drag->atom(QXcbAtom::D_QXCBATOM_WRAPPER(XdndActionCopy));
 
     if (actions.testFlag(Qt::MoveAction))
-        support_actions << drag->atom(QXcbAtom::XdndActionMove);
+        support_actions << drag->atom(QXcbAtom::D_QXCBATOM_WRAPPER(XdndActionMove));
 
     if (actions.testFlag(Qt::LinkAction))
-        support_actions << drag->atom(QXcbAtom::XdndActionLink);
+        support_actions << drag->atom(QXcbAtom::D_QXCBATOM_WRAPPER(XdndActionLink));
 
     // 此处不能直接 return ,因为一个拖拽包含多种 actions 后再次拖拽单一 action 时需要将 property 更新, 否则单一 actoin 的拖拽会被强行改成
     // 上一次的 actions, 导致某些小问题 (比如文管的拖拽行为不一致的问题)
     //    if (support_actions.size() < 2)
     //        return;
-
+#if QT_VERSION <= QT_VERSION_CHECK(6, 2, 4)
     xcb_change_property(drag->xcb_connection(), XCB_PROP_MODE_REPLACE, drag->connection()->clipboard()->m_owner,
-                        drag->atom(QXcbAtom::XdndActionList), XCB_ATOM_ATOM, sizeof(xcb_atom_t) * 8,
+                        drag->atom(QXcbAtom::D_QXCBATOM_WRAPPER(XdndActionList)), XCB_ATOM_ATOM, sizeof(xcb_atom_t) * 8,
                         support_actions.size(), support_actions.constData());
+#else
+    xcb_change_property(drag->xcb_connection(), XCB_PROP_MODE_REPLACE, drag->connection()->clipboard()->m_requestor, //TODO: m_ower deleted, replaced by m_requestor ?
+                        drag->atom(QXcbAtom::D_QXCBATOM_WRAPPER(XdndActionList)), XCB_ATOM_ATOM, sizeof(xcb_atom_t) * 8,
+                        support_actions.size(), support_actions.constData());
+#endif
     xcb_flush(drag->xcb_connection());
 }
 
+static void cursorThemePropertyChanged(xcb_connection_t *connection, const QByteArray &name, const QVariant &property, void *handle)
+{
+    Q_UNUSED(connection);
+    Q_UNUSED(name);
+    Q_UNUSED(property);
+    Q_UNUSED(handle)
+
+    QMetaObject::invokeMethod(qApp, [](){
+        for (const auto window : qApp->allWindows()) {
+            auto cursor = window->cursor();
+            if (window->screen() && window->screen()->handle() && window->screen()->handle()->cursor())
+                overrideChangeCursor(window->screen()->handle()->cursor(), &cursor, window);
+        }
+    }, Qt::QueuedConnection);
+}
+
 void DPlatformIntegration::initialize()
 {
     // 由于Qt很多代码中写死了是xcb，所以只能伪装成是xcb
@@ -1033,7 +1050,11 @@ void DPlatformIntegration::initialize()
     }
 
 #ifdef Q_OS_LINUX
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    m_eventFilter = new XcbNativeEventFilter(connection());
+#else
     m_eventFilter = new XcbNativeEventFilter(defaultConnection());
+#endif
     qApp->installNativeEventFilter(m_eventFilter);
 
     if (!qEnvironmentVariableIsSet("DXCB_DISABLE_HOOK_CURSOR")) {
@@ -1126,6 +1147,9 @@ void DPlatformIntegration::initialize()
             });
         }
     }
+
+    // Qt中同样监听了这个属性的变化，但是只刷新了光标上下文却没有更新当前窗口的光标，无法做到光标的实时变化，所以加了该逻辑额外处理
+    xSettings()->registerCallbackForProperty("Gtk/CursorThemeName", cursorThemePropertyChanged, nullptr);
 }
 
 #ifdef Q_OS_LINUX
@@ -1222,4 +1246,36 @@ void DPlatformIntegration::inputContextHookFunc()
 //    }
 }
 
+void DPlatformIntegration::sendEndStartupNotifition()
+{
+    QByteArray message, startupid;
+    if (QPlatformNativeInterface *ni = QGuiApplication::platformNativeInterface())
+        startupid = (const char *)ni->nativeResourceForIntegration(QByteArrayLiteral("startupid"));
+
+    if (startupid.isEmpty())
+        return ;
+
+    message =  QByteArrayLiteral("remove: ID=") + startupid;
+    xcb_client_message_event_t ev;
+    ev.response_type = XCB_CLIENT_MESSAGE;
+    ev.format = 8;
+    ev.type = xcbConnection()->atom(QXcbAtom::D_QXCBATOM_WRAPPER(_NET_STARTUP_INFO_BEGIN));
+    ev.sequence = 0;
+    ev.window = xcbConnection()->rootWindow();
+    int sent = 0;
+    int length = message.length() + 1; // include NUL byte
+    const char *data = message.constData();
+    do {
+        if (sent == 20)
+            ev.type = xcbConnection()->atom(QXcbAtom::D_QXCBATOM_WRAPPER(_NET_STARTUP_INFO));
+
+        const int start = sent;
+        const int numBytes = qMin(length - start, 20);
+        memcpy(ev.data.data8, data + start, numBytes);
+        xcb_send_event(xcbConnection()->xcb_connection(), false, xcbConnection()->rootWindow(), XCB_EVENT_MASK_PROPERTY_CHANGE, (const char *) &ev);
+
+        sent += numBytes;
+    } while (sent < length);
+}
+
 DPP_END_NAMESPACE
